@@ -10,17 +10,20 @@
 
 
 
+from math import log2
+
+
 class IPTool(object):
     int2ip = lambda x: '.'.join([str(int(x / (256 ** i) % 256)) for i in range(3, -1, -1)])
     ip2int = lambda x: sum([256 ** j * int(i) for j, i in enumerate(x.split('.')[::-1])])
     @staticmethod
-    def get_ip_mask(ip_string):
+    def get_ip_mask(ip_string:str):
         ipmask = ip_string.split("/")
         ip = IPTool.ip2int(ipmask[0] if len(ipmask) else "0.0.0.0")
         mask = int(ipmask[1] if len(ipmask)>1 else "32")
         return ip,mask
     @staticmethod
-    def net_range(ip_string):
+    def net_range(ip_string:str):
         ip,mask = IPTool.get_ip_mask(ip_string)
         start = (ip >> (32-mask) << (32-mask)) +1
         end = start + 2**(32-mask) -2
@@ -46,6 +49,51 @@ class IPTool(object):
             return True
         return False
 
+    @staticmethod
+    def get_ip_segments(start:int,end:int)->list:
+        '''
+        _summary_
+
+        Args:
+            start (int): _description_
+            end (int): _description_
+
+        Returns:
+            list: [(start,mask)]
+
+        >>> IPTools.get_ip_segments(0,1023)
+        [(0, 22)]
+        >>> IPTools.get_ip_segments(1,1024)
+        [(1, 32), (2, 31), (4, 30), (8, 29), (16, 28), (32, 27), (64, 26), (128, 25), (256, 24), (512, 23), (1024, 32)]
+
+        '''
+        ipmasks = []
+        if end==start:
+            return [(start,32)]
+        bits = int(log2(end-start+1))
+        for maskbits in range(bits,-1,-1):
+            if start >>maskbits <<maskbits == start :
+                ipmasks.append((start,32-maskbits))
+                break
+        if not end==start+2**maskbits-1:
+            ipmasks.extend(IPTool.get_ip_segments(start+2**maskbits,end))
+        return ipmasks
+
+    @staticmethod
+    def dedup(ip_segments:list)->list:
+        deduped = []
+        ip_segments.sort(key=lambda x:x[1])
+        for ip_seg in ip_segments:
+            is_in = False
+            for deduped_seg in deduped:
+                if ip_seg[0]>=deduped_seg[0] and (ip_seg[0]+2**(32-ip_seg[1]))<=(deduped_seg[0]+2**(32-deduped_seg[1])):
+                    is_in = True
+                    break  # have
+            if not is_in:
+                deduped.append(ip_seg)
+        return deduped    
+            
+        
 
 class Host(object):
     def __init__(self,id:str,ip:str="0.0.0.0/0") -> None:
@@ -229,9 +277,16 @@ class Router(object):
         return self
 
     def del_neighbor(self,neighbor,both:bool=True):
+        '''
+        delete neighbor router
+
+        Args:
+            neighbor (Router): neighbor router
+            both (bool, optional): delete neighbor in both side if both == True. Defaults to True.
+        '''
         router_id = neighbor.router_id
         if router_id in self.neighbors.keys():
-            self.pop(router_id)
+            self.neighbors.pop(router_id)
             self.del_neighbor_route_entrys(neighbor)
         if both:
             neighbor.del_neighbor(self,False)
